# -*- coding: utf-8 -*-

import requests

from .response import Response


class RequestClient:

    def __init__(self, base_url, token):
        self.base_url = base_url
        self.token = token
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': 'Bearer %s' % self.token
        })

    def __make_url(self, path):
        return '{}/{}'.format(self.base_url, path)

    def get(self, path, headers={}):
        response = self.session.get(self.__make_url(path), headers=headers)
        return Response('GET', response.status_code, response.text)

    def post(self, path, headers={}):
        response = self.session.post(self.__make_url(path), headers=headers)
        return Response('POST', response.status_code, response.text)

    def put(self, path, headers={}):
        response = self.session.put(self.__make_url(path), headers=headers)
        return Response('PUT', response.status_code, response.text)

    def delete(self, path, headers={}):
        response = self.session.get(self.__make_url(path), headers=headers)
        return Response('DELETE', response.status_code, response.text)

    def post_multipart(self, path, parts={}, headers={}):
        files = {k: (v['filename'], v['content'], v['content-type'])
                 for k, v in parts.items()}
        response = self.session.post(self.__make_url(path), headers=headers,
                                     files=files)
        return Response('POST', response.status_code, response.text)



# -*- coding: utf-8 -*-

import json

from .account import Account
from .error import DQError
from .job import Job, JobReport
from .request_client import RequestClient
from .response import from_response


class DQClient:

    def __init__(self, url, user, token, port=9001, api_ver='v1'):
        self.url = '{}/api/{}'.format(url, api_ver)
        self.port = port
        self.user = user
        self.token = token
        self.request_client = RequestClient(self.url, self.token)

    @from_response(Job)
    def list_jobs(self):
        """ GET 'https://app.dataquality.pl/api/v1/jobs' """
        return self.request_client.get('/jobs')

    @from_response(Job)
    def submit_job(self, config, input_data=None, input_file=None,
                   input_file_encoding='utf-8'):
        """ POST 'https://app.dataquality.pl/api/v1/jobs' """
        parts = {
            'config': {
                'filename': None,
                'content-type': 'application/json',
                'content': json.dumps(config.data())
            },
            'file': {
                'filename': 'tmp.csv',
                'content-type': 'text/csv',
            }
        }
        if input_data:
            parts['file']['content'] = str(input_data)
        else:
            parts['file']['content'] = open(input_file, 'r',
                                            encoding=input_file_encoding)

        return self.request_client.post_multipart('/jobs', parts=parts)

    def job_state(self, job_id):
        """ GET 'https://app.dataquality.pl/api/v1/jobs/{{job_id}}/status' """
        @from_response()
        def get_response():
            return self.request_client.get('/jobs/{}/status'.format(job_id))
        response = get_response()
        if response is None:
            return None
        return response['status']

    @from_response(JobReport)
    def job_report(self, job_id):
        """ GET 'https://app.dataquality.pl/api/v1/jobs/{{job_id}}' """
        return self.request_client.get('/jobs/{}'.format(job_id))

    def job_results(self, job_id, out_file=None):
        """ GET 'https://app.dataquality.pl/api/v1/jobs/{{job_id}}/result' """
        response = self.request_client.get('/jobs/{}/result'.format(job_id))
        if not response.is_ok():
            raise DQError(status=response.status, message=response.content)
        if out_file:
            try:
                file = open(out_file, 'w')
                file.write(response.content)
                file.close()
            except OSError:
                print("No such file or directory.")

    def delete_job(self, job_id):
        """ DELETE 'https://app.dataquality.pl/api/v1/jobs/{{job_id}}' """
        return self.request_client.delete('/jobs/{}'.format(job_id))

    def cancel_job(self, job_id):
        """ PUT 'https://app.dataquality.pl/api/v1/jobs/{{job_id}}/stop' """
        return self.request_client.put('/jobs/{}/stop'.format(job_id))

    @from_response(Account)
    def account_status(self):
        """ GET 'https://app.dataquality.pl/api/v1/account/status' """
        return self.request_client.get('/account/status')



# -*- coding: utf-8 -*-


class Account:

    def __init__(self, email, balance, credit, total_records):
        self.email = email
        self.balance = balance,
        self.credit = credit,
        self.total_records = total_records

    def __repr__(self):
        return str(vars(self))



from time import sleep

from dq import DQClient, JobConfig

JOB_TEMPLATE = {
    "job_name": "marcin 123",
    "input_format": {
        "field_separator": ",",
        "text_delimiter": "",
        "header": 1
    },
    "input_columns": [{
        "no": 0,
        "name": "Name",
        "function": "KOD_POCZTOWY"
    }, {
        "no": 1,
        "name": "Code",
        "function": "MIEJSCOWOSC"
    }, {
        "no": 2,
        "name": "Default City",
        "function": "ULICA_NUMER_DOMU_I_MIESZKANIA"
    }, {
        "no": 3,
        "name": "Default postcode",
        "function": "PRZEPISZ"
    }, {
        "no": 4,
        "name": "Country zone",
        "function": "PRZEPISZ"
    }],
    "extend": {
        "teryt": 0,
        "gus": 0,
        "geocode": 0,
        "building_info": 0,
        "diagnostic": 1,
        "area_characteristic": 0
    }
}

PLAY_JOB_TEMPLATE = {
    "job_name": "marcin 123",
    "input_format": {
        "format": "csv",
        "field_separator": ";",
        "text_delimiter": "\"",
        "code_page": "utf-8",
        "header": 1
    },
    "input_columns": [
        {"no": 0, "name": "id", "function": "PRZEPISZ"},
        {"no": 1, "name": "request", "function": "CALY_ADRES"}
    ],
    "extend": {
        "teryt": 0,
        "gus": 1,
        "geocode": 1,
        "building_info": 0,
        "diagnostic": 1,
        "area_characteristic": 0
    }
}

PLAY_INPUT_DATA = """"ID";"REQUEST"
6876;"34-404, PYZÓWKA, PODHALAŃSKA 100"
"""

INPUT_FILE = """
id, adres,test1,test2,test3,test4,test5,test6
1, 222 bbb,aaaa,bbbb,cccc,dddd,eeeee,ffffff
2, 222 bbb,aaaa,bbbb,cccc,dddd,eeeee,ffffff
3, 222 bbb,aaaa,bbbb,cccc,dddd,eeeee,ffffff
4, 222 bbb,aaaa,bbbb,cccc,dddd,eeeee,ffffff
5, 222 bbb,aaaa,bbbb,cccc,dddd,eeeee,ffffff
6, 222 bbb,aaaa,bbbb,cccc,dddd,eeeee,ffffff
"""


def main():
    dq = DQClient('https://app.dataquality.pl', user='test2@3e.pl', token='imjn8faed2g8bg1xy0sc06wbc7qhewqvevf16z01')

    # dq.list_jobs()
    # print(dq.job_status('408b7a3b-302e-4ecd-9363-11a4c36db7ff'))
    # print(dq.job_result('408b7a3b-302e-4ecd-9363-11a4c36db7ff'))
    # dq.job_result_data('7750b6fc-74bb-4072-892c-81afda79e330')
    print(dq.account_status())
    print(dq.list_jobs())
    print(dq.job_report('2dfafd80-7986-42d1-96be-06260aa32b6f'))
    print(dq.job_state('3f14e25e-9f6d-41ff-a4cb-942743a37b73'))
    print(dq.job_report('3f14e25e-9f6d-41ff-a4cb-942743a37b73'))

    #

    # usuwanie, stopowanie i pobieranie wyników nieistniejącego joba
    # print dq.delete_job('7750b6fc-74bb-4072-892c-81afda79e331')
    # print dq.stop_job('7750b6fc-74bb-4072-892c-81afda79e331')
    # print dq.job_result('7750b6fc-74bb-4072-892c-81afda79e331')


    # tworzenie joba


    # res_method, res_code, res_content = dq.submit_job(JOB_TEMPLATE, input_data=INPUT_FILE)
    # res_method, res_code, res_content = dq.submit_job(JOB_TEMPLATE, input_data=INPUT_FILE)
    # res_method, res_code, res_content = dq.submit_job(JOB_TEMPLATE, input_file='samples/other_data0.csv')


    # resp = dq.submit_job(PLAY_JOB_TEMPLATE, input_data=PLAY_INPUT_DATA)
    # dq.submit_job(JOB_TEMPLATE, input_file='samples/telco_data2.csv')
    # print(resp)

    jobs = []

    for i in range(5):
        job_config = JobConfig('job {}'.format(i))
        job_config.input_format(field_separator=";", text_delimiter="\"", has_header=True)
        job_config.input_column(0, name='id', function='PRZEPISZ')
        job_config.input_column(1, name='request', function='CALY_ADRES')
        job_config.extend(gus=True, geocode=True, diagnostic=True)
        job = dq.submit_job(job_config, input_file='samples/telco_data2.csv',
                            input_file_encoding='windows-1250')
        print(job)

        print(dq.job_state(job.id))
        print(dq.job_report(job.id))

        jobs.append(job)

        if job is None:
            print('something went wrong')
            return
        else:
            print('Job created with id: %s' % job.id)
            # sleep(0.01)

            # zwiększyć czas

    for job in jobs:
        print('checking job {}'.format(job.id))
        for x in range(0, 100):
            state = dq.job_state(job.id)
            if state == 'FINISHED':
                print('finished jobid: {}'.format(job.id))
                break
            print('state %s: %s' % (x, state))
            sleep(5.0)

        print('Getting job results ...')
        print(dq.job_report(job.id))
        dq.job_results(job.id, 'ouput.csv')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from .client import DQClient
from .job import JobConfig

__author__ = """Mikołaj Olszewski"""
__email__ = 'mikolaj.olszewski@algolytics.pl'
__version__ = '0.1.0'

__all__ = ['DQClient', 'JobConfig']



# -*- coding: utf-8 -*-

import json

from .error import DQError


class Response:

    def __init__(self, method, status, content):

        self.method = method
        self.status = status
        self.content = content

    def is_ok(self):
        return self.status and int(self.status) < 400

    def json(self):
        return self.result

    def object(self):
        return json.loads(self.content)

    def __repr__(self):
        return "%s %s" % (self.status, self.content)


class from_response():

    def __init__(self, clazz=None):
        self.clazz = clazz

    def __call__(self, f):
        def wrapper(*args, **kwargs):
            response = f(*args, **kwargs)
            if response.status == 404:
                return None
            if not response.is_ok():
                raise DQError(status=response.status, message=response.content)
            obj = response.object()
            if isinstance(obj, list):
                return [self.__to_object(item) for item in obj]
            return self.__to_object(obj)
        return wrapper

    def __to_object(self, obj):
        if self.clazz is None:
            return obj
        return self.clazz(**obj)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'requests == 2.11.1'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='dq-client',
    version='0.1.0',
    description="Python library which allows to use http://dataquality.pl in easy way.",
    long_description=readme + '\n\n' + history,
    author="Mikołaj Olszewski",
    author_email='mikolaj.olszewski@algolytics.pl',
    url='https://github.com/Algolytics/dq_client',
    packages=[
        'dq',
    ],
    package_dir={'dq': 'dq'},
    include_package_data=True,
    install_requires=requirements,
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords=['dataquality', 'dq', 'dq-client'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# -*- coding: utf-8 -*-


class Job:

    def __init__(self, job_id, name, status, start_date, end_date,
                 source_records, processed_records, price):
        self.id = job_id
        self.name = name
        self.status = status
        self.start_date = start_date
        self.end_date = end_date
        self.source_records = source_records
        self.processed_records = processed_records
        self.price = price

    def __repr__(self):
        return str(vars(self))


class JobReport:

    def __new__(cls, *args, **kwargs):
        if 'code' in kwargs:
            return None
        return super().__new__(cls)

    def __init__(self, quality_issues, quality_names, results):
        self.quality_issues = quality_issues
        self.quality_names = quality_names
        self.results = results

    def __repr__(self):
        return str(vars(self))


class JobConfig:

    def __init__(self, name):
        self.name = name
        self._input_format = {
            "field_separator": ",",
            "text_delimiter": "\"",
            "header": 1
        }
        self._input_columns = {}
        self._extend = {
            "teryt": 0,
            "gus": 0,
            "geocode": 0,
            "building_info": 0,
            "diagnostic": 0,
            "area_characteristic": 0
        }

    def input_format(self, field_separator=",", text_delimiter="\"",
                     has_header=True):
        self._input_format["field_separator"] = field_separator
        self._input_format["text_delimiter"] = text_delimiter
        self._input_format["header"] = self.__boolean_to_num(has_header)

    def input_column(self, idx, name, function):
        self._input_columns[idx] = {"no": idx, "name": name,
                                    "function": function}

    def extend(self, teryt=False, gus=False, geocode=False,
               building_info=False, diagnostic=False,
               area_characteristic=False):
        self._extend["teryt"] = self.__boolean_to_num(teryt)
        self._extend["gus"] = self.__boolean_to_num(gus)
        self._extend["geocode"] = self.__boolean_to_num(geocode)
        self._extend["building_info"] = self.__boolean_to_num(building_info)
        self._extend["diagnostic"] = self.__boolean_to_num(diagnostic)
        self._extend["area_characteristic"] = \
            self.__boolean_to_num(area_characteristic)

    @staticmethod
    def __boolean_to_num(value):
        return 1 if value else 0

    def data(self):
        return {
            "job_name": self.name,
            "input_format": self._input_format,
            "input_columns": list(self._input_columns.values()),
            "extend": self._extend
        }
